import sys
#from promoterExtract import promoter

def sub_usage(args):
    if len(args) == 1:
        print("          \033[1;33;40m\nUsage  :\033[1m\033[1;35;40m  %s\033[1m" %s (args[0]) )
    elif len(args) == 2:
        print("          \033[1;35;40m%s\033[0m    \033[1;32;40m%s\033[0m" % (args[0], args[1]) )

def main_usage():
    print("\n\033[1;33;40mProgram: \033[0m\033[1;35;40m get_promoter \033[1;31;40m(pipeline for promoter extract)\033[0m")
    print("\033[1;33;40mVersion: \033[0m\033[1;32;40m 0.9.5.7\033[0m")
    print("\033[1;33;40mContact: \033[0m\033[1;32;40m Sitao Zhu <zhusitao1990@163.com>\033[0m")
    print("\033[1;33;40mUsage  : \033[0m\033[1;35;40m get_promoter\033[0m \033[1;31;40m<command>\033[0m")
    print("\033[1;33;40mCommand: \033[0m")
    sub_usage(["create ", "create database for GTF or GFF"])
    sub_usage(["extract", "extract promoter for genome or gene"])
    sys.exit(1)


def main():
    if len(sys.argv) == 1:
        main_usage()
    elif len(sys.argv) >= 2:
        if sys.argv[1] in ['create','extract']:
            # import 就执行promoter()
            from promoterExtract import promoter
        else:
            main_usage()

if __name__ == "__main__":
    main()



import re,os
import sys
import gzip
import matplotlib.pyplot as plt
from matplotlib_venn import venn2
import pandas as pd

"""
Author: Zhu Sitao
Date : 2018-3-21
Dest: a DNA fasta class; used in python 3.6
VCF is a text file format (most likely stored in a compressed manner). 
It contains meta-information lines, a header line, and then data lines 
each containing information about a position in the genome.
http://www.internationalgenome.org/wiki/Analysis/vcf4.0
http://samtools.github.io/hts-specs/VCFv4.2.pdf
"""

class Header(object):
	""" a class for vcf header """
	def __init__(self,vcfpath):
		self.path = vcfpath
		self.info_pattern = re.compile(r'''\#\#INFO=<
		            		ID=(?P<id>[^,]+),\s*
		            		Number=(?P<number>-?\d+|\.|[AGR])?,\s*
		            		Type=(?P<type>Integer|Float|Flag|Character|String),\s*
		            		Description="(?P<desc>[^"]*)"
		            		(?:,\s*Source="(?P<source>[^"]*)")?
		            		(?:,\s*Version="?(?P<version>[^"]*)"?)?
		            		>''', re.VERBOSE)
		self.filter_pattern = re.compile(r'''\#\#FILTER=<
		            		ID=(?P<id>[^,]+),\s*
		            		Description="(?P<desc>[^"]*)"
		            		>''', re.VERBOSE)
		self.alt_pattern = re.compile(r'''\#\#ALT=<
		            		ID=(?P<id>[^,]+),\s*
		            		Description="(?P<desc>[^"]*)"
		            		>''', re.VERBOSE)
		self.format_pattern = re.compile(r'''\#\#FORMAT=<
		  			ID=(?P<id>.+),\s*
		            		Number=(?P<number>-?\d+|\.|[AGR]),\s*
		            		Type=(?P<type>.+),\s*
		            		Description="(?P<desc>.*)"
		            		>''', re.VERBOSE)
		self.contig_pattern = re.compile(r'''\#\#contig=<
		            		ID=(?P<id>[^>,]+)
		            		(,.*length=(?P<length>-?\d+))?
		            		.*
		            		>''', re.VERBOSE)
		self.head_pattern = re.compile(r'\#CHROM\s*POS', re.VERBOSE)
		self.meta_pattern = re.compile(r'''##(?P<key>.+?)=(?P<val>.+)''')
		self.referPatter = re.compile(r'##reference=file:\/\/(\S+)')
		self.samples = self.getSampleID()
	def readHeader(self):
		""" read header of vcf"""
		with open(self.path,'r') as F:
			for line in F:
				if line.startswith('#') and len(line) !=0:
					yield line
				else:
					break
	def getChromLength(self):
		""" Return a chromosome length dict """
		chromLenDict = dict()
		for line in self.readHeader():
			match = self.contig_pattern.match(line)
			if match:
				chromLenDict[match.group('id')] = match.group('length')
		return chromLenDict

	def getReferPath(self):
		""" Return reference genome path """
		for line in self.readHeader():
			match = self.referPatter.search(line)
			if match:
				return match.group(1)


	def getReferLength(self):
		""" Calculate reference genome length """
		total_length = 0
		chromLenDict = self.getChromLength()
		for key in chromLenDict.keys():
			total_length += int(chromLenDict[key])
		return total_length

	def getSampleID(self):
		""" Return population vcf file sample ids """
		sampleList = []
		for line in self.readHeader():
			match = self.head_pattern.match(line)
			if match:
				temp = line.strip().split()
				for sample in range(9,len(temp)):
					sampleList.append(temp[sample])
		return sampleList


	def header_to_file(self,out_path):
		""" write a header to a file """
		F = open(out_path,'w')
		for line in self.readHeader():
			F.write(line)
		F.close()

class Recorder(object):
	""" a recorder for vcf """
	def __init__(self, CHROM, POS, ID, REF, ALT, QUAL, FILTER, INFO,FORMAT,CALLS):
		#: A ``str`` with the chromosome name
		self.CHROM = CHROM
		#: An ``int`` with a 1-based begin position
		self.POS = POS
		#: An ``int`` with a 0-based begin position
		self.begin = eval(POS) - 1
		#: An ``int`` with a 0-based end position
		self.end = None  # XXX
		#: A list of the semicolon-separated values of the ID column
		#self.ID = list(ID)
		self.ID = ID
		#: A ``str`` with the REF value
		self.REF = REF
		#: A list of alternative allele records of type :py:class:`AltRecord`
		#self.ALT = list(ALT)
		self.ALT = ALT
		#: The quality value, can be ``None``
		self.QUAL = QUAL
		#: A list of strings for the FILTER column
		self.FILTER = FILTER
		#: An OrderedDict giving the values of the INFO column, flags are
		#: mapped to ``True``
		self.INFO = INFO
		#: A list of strings for the FORMAT column.  Optional, must be given if
		#: and only if ``calls`` is also given.
		self.FORMAT = FORMAT or ()
		#: A list of genotype :py:class:`Call` objects.  Optional, must be given if
		#: and only if ``FORMAT`` is also given.
		self.CALLS = CALLS or ()


	def is_snv(self):
		""" Return ture is a snv"""
		return len(self.REF) == 1 and all(a.type == "SNV" for a in self.ALT)
	def record_to_line(self):
		line = [self.CHROM,self.POS,self.ID,self.REF,self.ALT,self.QUAL,self.FILTER,self.INFO,self.FORMAT]
		for call in self.CALLS:
			line.append(call)
		return "\t".join(line)

	def filter_by_quality(self, cutoff=30):
		""" filter vcf by information """
		if self.QUAL > cutoff:
			return self.record_to_line()

	def filter_by_depth(self,down,up):
		""" filter vcf by depth <4 or >200"""
		info = Info(self.INFO)
		if info.DP <= up or info.DP >= down:
			return self.record_to_line()


class AltRecord(object):
	""" A class  for alternative allele record """
	def __init__(self,allele=None):
		self.ale = allele
		self.ale_list= self.allele_list()
	def allele_list(self):
		""" return all allele item include the ref,
		which corresponding to the genotype of each sample call"""
		if ',' in self.ale:
			return self.ale.split(',')
		else:
			return self.ale



class Call(object):
	""" a class for single call """
	def __init__(self, sample, data):
		#: the name of the sample for which the call was made
		self.sample = sample
		#: an OrderedDict with the key/value pair information from the
		#: call's data
		self.data = data
		#: the allele numbers (0, 1, ...) in this calls or None for no-call
		self.gt_alleles = self.parser_call()
	def parser_call(self):
		genotype_index = self.data.split(":")[0].split("/")
		return genotype_index

class Info(object):
	""" a class for vcf INFO column """
	def __init__(self,data):
		# An integer,Allele count in genotypes, for each ALT allele, in the same order as listed
		self.data = data
		self.DP =self.depth()
		self.AC = self.allele_count()
		self.AF = self.allele_frequency()
	def depth(self):
		""" return a Combined depth across samples"""
		DP_pattern = re.compile(r'DP=(?P<value>\d+);')
		DP_value = DP_pattern.search(self.data).group('value')
		return DP_value
	def allele_count(self):
		""" return a allele count """
		AC_pattern = re.compile(r'AC=(?P<value>\d+);')
		AC_value = AC_pattern.search(self.data).group('value')
		return AC_value
	def allele_frequency(self):
		""" return an allele frequency """
		AF_pattern = re.compile(r'AF=(?P<value>-?\d+\.?\d*e?-?\d*)?;')
		AF_value = AF_pattern.search(self.data).group('value')
		return AF_value








class VCF(object):
	""" VCF class """
	def __init__(self,vcfpath):
		self.path = vcfpath
		self.basename = os.path.basename(vcfpath)

	def readVCF(self):
		""" Return a vcf file line by line """
		if not os.path.exists(self.path):
			sys.exit("%s file not exist\n"%(self.path))
		if self.path.endswith(".gz"):
			F = gzip.open(self.path)
		else:
			F = open(self.path,'r')
		for line in F:
			if not line.startswith('#'):
				array = line.strip().split()
				CHROM,POS,ID,REF,ALT,QUAL,FILTER,INFO,FORMAT= array[0:9]
				CALLS = array[9:len(array)] # a list
				recorder = Recorder(CHROM,POS,ID,REF,ALT,QUAL,FILTER,INFO,FORMAT,CALLS)
				yield recorder
		F.close()

	def __len__(self):
		""" Return numbers of items in self """
		totalLength = 0
		for i in self.readVCF():
			totalLength += 1
		return totalLength

	def Homo_Hete_Sites_Stat(self):
		""" Stat homozygosis and heterozygosis variation sites"""
		All_samples = dict()
		sampleids = Header(self.path).samples
		for sample in sampleids:
			All_samples[sample] = [0,0,0,0] # HomozygousSitesStat, heterozygousSitesStat, missingSitesStat, nonSitesStat
		for recorder in self.readVCF():
			alleles = AltRecord(allele=recorder.ALT).ale_list
			for i in range(len(sampleids)):
				GT = Call(sampleids[i],recorder.CALLS[i]).gt_alleles
				if GT[0] == "." and GT[1] == ".":
					All_samples[sampleids[i]][3] += 1
					#nonSitesStat += 1
				elif "*" in GT[0] or "*" in GT[1]:
					All_samples[sampleids[i]][2] += 1
					#missingSitesStat += 1
				else:
					if GT[0] == GT[1]:
						All_samples[sampleids[i]][0] += 1
						#HomozygousSitesStat += 1
					else:
						All_samples[sampleids[i]][1] += 1
						#heterozygousSitesStat += 1
		#Table header
		print ("Type\t"+"\t".join(sampleids))
		Type_sites = ["Homozygous   Sites", "Heterozygous Sites", "MissingSites(contain *)", "NonSites(contain .)"]
		for type in Type_sites:
			output = [type]
			for sample in sampleids:
				output.append(str(All_samples[sample][Type_sites.index(type)]))
			print ("\t".join(output))
			output = []

	def getHomoSites(self,HomoSites_outpath):
		""" Return homozygosis sites , including header line and variation line"""
		HomoSites =open(HomoSites_outpath,'w')
		# header
		header = Header()
		for line in header.readHeader(self.path):
			HomoSites.write(line)
		# record
		for recorder in self.readVCF():
			genotype = recorder.CALLS[0]
			GT = genotype.split(":")[0].split("/")
			## homo
			if GT[0] == "." and GT[1] == ".":
				continue
			elif "*" in GT[0] or "*" in GT[1]:
				continue
			else:
				if GT[0] == GT[1]:
					line = recorder.record_to_line()
					HomoSites.write(line+"\n")
		HomoSites.close()


	def getHeteSites(self,HeteSites_outpath):
		"""Return heterozygosis sites , including header and variation line """
		HeteSites = open(HeteSites_outpath, 'w')
		header = Header()
		for line in header.readHeader(self.path):
			HeteSites.write(line)
		for recorder in self.readVCF():
			genotype = recorder.CALLS[0]
			GT = genotype.split(":")[0].split("/")
			## homo
			if GT[0] == "." and GT[1] == ".":
				continue
			elif "*" in GT[0] or "*" in GT[1]:
				continue
			else:
				if GT[0] != GT[1]:
					line = recorder.record_to_line()
					HeteSites.write(line+"\n")
		HeteSites.close()

	def reduceNonSites(self):
		pass

	def extract_snp(self,snp_path):
		""" extract snp into a new vcf """
		F = open(snp_path,'w')
		for recorder in self.readVCF():
			if len(recorder.REF) == len(recorder.ALT):
				F.write(recorder.record_to_line())
		F.close()



	def extract_indel(self,indel_path):
		""" extract indel into a new vcf """
		F =open(indel_path,'w')
		for recorder in self.readVCF():
			if ',' in recorder.ALT:
				if len(recorder.REF) != len(recorder.ALT.split(',')[0]) :
					F.write(recorder.record_to_line())
			else:
				if len(recorder.REF) != len(recorder.ALT):
					F.write(recorder.record_to_line())
		F.close()

	def stat_indel_length(self):
		""" return a indel length dict for draw indel length histogram """
		indel_length = {"delete":{},"insert":{}}

		for recorder in self.readVCF():
			if len(recorder.REF) == 1 and len(recorder.ALT) == 1:
				continue
			else:
				if len(recorder.ALT) - len(recorder.REF) > 0: # insert
					delete_lengthinsert_length = len(recorder.ALT) - len(recorder.REF)
					if insert_length in insert_length["insert"].keys():
						indel_length["insert"][length] += 1
					else:
						indel_length["insert"][length] = 1
				else:
					delete_length = len(recorder.REF) - len(recorder.ALT)
					if delete_length in delete_length["delete"].keys():
						indel_length["delete"][length] += 1
					else:
						indel_length["delete"][length] = 1
		return indel_length
	def indel_length_plot(self):
		insert_df = pd.DataFrame(self.indel_length_plot()['insert'].items())
		insert_df.columns = ['length','count']
		delete_df = pd.DataFrame(self.indel_length_plot()['delete'].items())
		base_plot = (ggplot(insert_df,aes(x='length',y='count')))+geom_bar(stat='identity')
		




	def vcf2genotype(self,genotype_outpath):
		""" Transformate population vcf to genotype """
		IUPAC = {'-' : '-', 'AA' : 'A', 'TT' : 'T', 'CC' : 'C', 'GG' : 'G', 'AG' : 'R',
				'GA' : 'R', 'CT' : 'Y', 'TC' : 'Y', 'AC' : 'M', 'CA' : 'M', 'GT' : 'K',
				'TG' : 'K', 'GC' : 'S', 'CG' : 'S', 'AT' : 'W', 'TA' : 'W',
				'ATC' : 'H', 'ACT' : 'H', 'TAC' : 'H', 'TCA' : 'H', 'CAT' : 'H', 'CTA' : 'H',
				'GTC' : 'B', 'GCT' : 'B', 'CGT' : 'B', 'CTG' : 'B', 'TCG' : 'B', 'TGC' : 'B',
				'GAC' : 'V', 'GCA' : 'V', 'CGA' : 'V', 'CAG' : 'V', 'AGC' : 'V', 'ACG' : 'V',
				'GAT' : 'D', 'GTA' : 'D', 'AGT' : 'D', 'ATG' : 'D', 'TGA' : 'D', 'TAG' : 'D', 
				'ATCG' : 'N', 'ATGC' : 'N', 'ACTG' : 'N', 'ACGT' : 'N', 'AGTC' : 'N', 'AGCT' : 'N', 'TACG' : 'N',
				'TAGC' : 'N', 'TCAG' : 'N', 'TCGA' : 'N', 'TGAC' : 'N', 'TGCA' : 'N', 'CATG' : 'N', 'CAGT' : 'N',
				'CTAG' : 'N', 'CTGA' : 'N', 'CGAT' : 'N', 'CGTA' : 'N', 'GATC' : 'N', 'GACT' : 'N', 'GTAC' : 'N',
				'GTCA' : 'N', 'GCAT' : 'N', 'GCTA' : 'N'}
				#this hash was use for converting the genotype to a single letter according to th IUPAC.
		OUT = open(genotype_outpath,"w")
		header = "Chr\tPosi\tRef"
		for sample in self.getSampleID():
			sampleTab = "\t"+sample
			header += sampleTab
		OUT.write(header+"\n")

		for line in self.readVCF():
			temp,alts = [],[]
			if len(line) != 0 and not line.startswith("#"):
				temp = line.strip().split("\t")
				OUT.write(temp[0]+"\t"+temp[1]+"\t"+temp[3]+"\t") #output the chromosome number, positions and the allele of reference.
				if "," in temp[4]:
					alts = temp[4].split(',')
				else:
					alts.append(temp[4])
				alts.insert(0,temp[3])
				for num in range(9,len(temp)):
					geno = ''
					if re.compile(r'\.\/\.').match(temp[num]):
						geno = "-"
					else:
						aa = re.compile(r'(\d)\/(\d).*').match(temp[num])
						allele1 = alts[int(aa.group(1))]
						allele2 = alts[int(aa.group(2))]
						if "*" in allele1 or "*" in allele2:
							geno = "-"
						else:
							geno = allele1+allele2
					OUT.write(IUPAC[geno]+"\t")
				OUT.write("\n")
		OUT.close()


	def ts_tv(self):
		""" transitions and transversions for snp """
		ts,tv = 0,0
		ts_flag = ["AG","GA","CT","TC"]
		tv_flag = ["AC","CA","GT","TG","GC","CG","AT","TA"]
		for recorder in self.readVCF():
			ref = recorder.REF
			ale = recorder.ALT
			if ref.upper()+ale.split(',')[0].upper() in ts_flag:
				ts += 1
			else:
				tv += 1
		return ts,tv,'%.6f'%(ts/float(tv))

	def compare_vcf(self,other):
		list1 = []
		list2 = []
		for record in self.readVCF():
			chrom,pos,ref,ale = record.CHROM,record.POS,record.REF,record.ALT
			list1.append(chrom+pos+ref+ale)
		for record in other.readVCF():
			chrom, pos, ref, ale = record.CHROM, record.POS, record.REF, record.ALT
			list2.append(chrom + pos + ref + ale)
		set1 = set(list1)
		set2 = set(list2)
		plt.figure(figsize=(4,4))
		venn2(subsets = [set1,set2],set_labels=(self.basename,other.basename),set_colors=('r','g'))
		plt.savefig("{t1}_vs_{t2}.png".format(t1=self.basename,t2=other.basename))








import optparse
import sys

IUPAC_bases2dna = dict({"AA":"A","CC":"C","GG":"G","TT":"T",
               "AG":"R","CT":"Y","AC":"M","GT":"K",
               "AT":"W","CG":"S","CGT":"B","AGT":"D",
               "ACT":"H","ACG":"V","ACGT":"N"})
FASTQ_quality_string = '''!"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~'''

VERSION = '17.04.24'
NAME = 'vcf2fastq'
descr = """
This program converts single sample vcf-files into fastq files. Fastq quality
values are taken from the sample field's genotype quality. Input vcf files
should be sorted by position.
This program was created to produce suitable input files for psmc fq2psmcfa
script (Li & Durbin 2011 Inference of Human Population History From Whole
Genome Sequence of A Single Individual. Nature 475(7357):493-496)
"""

print('\nRunning {0} v.{1}'.format(NAME, VERSION))

parser = optparse.OptionParser(description=descr)
parser.add_option('-i', '--input', help='path to an input file (vcf)',
                  type='string')
parser.add_option('-o', '--output_file', help='output file path',
                  type='string')
args = parser.parse_args()[0]

def vcf2fastq(in_path, out_path):
    in_handle = open(in_path)
    out_handle = open(out_path, 'w')

    prev_scaffold = None
    lines_processed = 0
    for line in in_handle:
        lines_processed += 1
        if lines_processed % 1000000 == 0:
            print ('{0} million vcf-file lines processed...'.format(lines_processed/1000000))
        # Skip empty and header lines
        line.strip()
        if line.startswith('##'): continue
        if not line: continue
        line = line.split('\t')
        if line[0].startswith('#'):
            if len(line) != 10:
                print ('Error! A single sample vcf file is expected as input ' \
                      'containing 10 columns, {0} columns found!'.format(len(line)))
            continue
        #Parse and write output
        scaffold = line[0]
        position = int(line[1])
        if prev_scaffold is None: # i.e. the first row
            sequence = []
            sequence_quality = []
        elif scaffold != prev_scaffold: # Write output when scaffold changes
            out_handle.write('@{0}\n'.format(prev_scaffold))
            out_handle.write('{0}\n+\n'.format(''.join(sequence)))
            out_handle.write('{0}\n'.format(''.join(sequence_quality)))
            sequence = []
            sequence_quality = []
        else:
            if position != (prev_position + 1): # Write 'n' character for gaps in sequence
                sequence += ['n']*(position-prev_position-1)
                sequence_quality += ['!'*(position-prev_position-1)]
            sequence.append(parse_genotype(line))
            # If genotype is missing, use the lowest quality GQ value
            if sequence[-1] == 'n':
                sequence_quality.append('!')
            else:
                sequence_quality.append(parse_genotype_quality(line))
        prev_scaffold = scaffold
        prev_position = position
    # Write the last sequence
    out_handle.write('@{0}\n'.format(prev_scaffold))
    out_handle.write('{0}\n+\n'.format(''.join(sequence)))
    out_handle.write('{0}\n'.format(''.join(sequence_quality)))
    in_handle.close()
    out_handle.close()
    print ('File conversion successfull!')


def parse_genotype(line):
    """Extracts the genotype from vcf file line"""
    gt_field = line[9].split('\t')[0]
    gt_field = gt_field.split(':')[0]
    if gt_field in ('.', './.', '.|.'):
        return 'n'
    if '/' in gt_field:
        gt_field = gt_field.split('/')
    elif '|' in gt_field:
        gt_field.split('|')
    else:
        gt_field = [gt_field]

    genotypes = []
    for gt in gt_field:
        if gt == '0':
            genotypes.append(line[3])
        else:
            try:
                genotypes.append(line[4].split(',')[int(gt) - 1])
            except ValueError:
                print ('Error! Odd genotype value on line:')
                print ('\t'.join(line))
                sys.exit(0)

    return IUPAC_bases2dna[''.join(sorted(genotypes))]


def parse_genotype_quality(line):
    """Extracts the genotype quality (GQ) from vcf file line"""
    sample_fields = line[9].split(':')
    try:
        GQ_index = line[8].split(':').index('GQ')
    except ValueError:
        print ('Error! GQ field was not found on line')
        sys.exit(0)
    try:
        try:
            # int(float()) to round down genotype quality
            return FASTQ_quality_string[int(float(sample_fields[GQ_index]))]
        except IndexError:
            return '~'
    except ValueError:
        # Genotype quality value is not integer on line
        print ('\t'.join(line))
        sys.exit(0)


vcf2fastq(args.input, args.output_file)


import os
import gffutils
import pandas as pd
from Bio import SeqIO
from Bio.SeqRecord import SeqRecord
import argparse


def create(args):
    fn = args.gff # gffutils.example_filename(gff_path)
    db = gffutils.create_db(fn, dbfn='gff.db', force=True, keep_order=True,\
        disable_infer_genes=True, disable_infer_transcripts=True,\
        merge_strategy='merge', sort_attribute_values=True)
    return db

def genome_dict(genome_fasta_path):
    genome = dict()
    for record in SeqIO.parse(genome_fasta_path, 'fasta'):
        genome[record.id] = record.seq
    return genome

def get_utr5(db, gene):
    for i in db.children(gene, featuretype='mRNA', order_by='start'):
        print(i)

def get_utr3(db, gene):
    pass


def extract(args):
    promoter_length = args.length
    utr_head_length = args.utr_head
    genome_path = args.genome
    gff_db_path = args.gff
    output_file = args.output
    genome = genome_dict(genome_path)
    # db = create_db(gff_path)
    db = gffutils.FeatureDB(gff_db_path, keep_order=True)
    index = 0
    promoter_seq = pd.DataFrame(columns=['GeneID','Chrom','Start','End','Strand','Promoter'])
    for f in db.all_features(featuretype='gene', order_by="seqid"):
        chrom = f.chrom
        geneid = f.id
        strand = f.strand
        if f.strand == "+":
            gene_seq = f.sequence(genome_path, use_strand=True)
            p_start = f.start - int(promoter_length) - 1
            if p_start < 0 :
                continue
            p_end = f.start + utr_head_length - 1
            promoter = genome[f.chrom][p_start:p_end]
            # print(promoter)
        elif f.strand == "-":
            gene_seq = f.sequence(genome_path, use_strand=True)
            p_start = f.end - utr_head_length
            p_end = f.end + int(promoter_length)
            promoter = genome[f.chrom][p_start:p_end].reverse_complement()
            # print(promoter)
        p_start_in_genome = p_start
        p_end_in_genome = p_end
        promoter_seq.loc[index] = [geneid,chrom,p_start_in_genome,p_end_in_genome,strand,promoter]
        index += 1
    promoter_seq.to_csv(output_file, sep=',', index=False)

parser = argparse.ArgumentParser()
subparsers = parser.add_subparsers(help='sub-command help')
# create subcommand 
parser_create = subparsers.add_parser('create', help='create annotation database')
parser_create.add_argument('-g', '--gff', type=str, help='genome annotation file')
parser_create.set_defaults(func=create)
# extract subcommand
parser_extract = subparsers.add_parser('extract', help='extract promoter in genome or gene')
parser_extract.add_argument('-l', '--length', type=int, help='promoter length before TSS')
parser_extract.add_argument('-g', '--gff', type=str, help='gff binary database created before')
parser_extract.add_argument('-u', '--utr_head', type=int, help='utr5 length after TSS')
parser_extract.add_argument('-f', '--genome', type=str, help='genome fasta')
parser_extract.add_argument('-o', '--output', type=str, help = 'output csv file path')
parser_extract.add_argument('-v', '--version', help = 'promoterExtract version', action = "store_true")
parser_extract.set_defaults(func=extract)
args = parser.parse_args()
print('runing ...')
args.func(args)
print('finished ...')



'''
if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('-l', '--length', type=int, help='promoter length before TSS')
    parser.add_argument('-u', '--utr_head', type=int, help='length after TSS')
    parser.add_argument('-f', '--genome', type=str, help='genome fasta')
    parser.add_argument('-g', '--gff', type=str, help='genome annotation file')
    parser.add_argument('-o', '--outdir', type=str, help='output directory')
    args = parser.parse_args()
    get_promoter(args.length, args.utr_head, args.genome, args.gff, args.outdir)
'''                                             



import re
from fasta import Fasta,fasta_record
"""
Author: Zhu Sitao
Date: 2018-3-21
Dest: The readGFF.py is use to read GFF3,Python version is 3.6
GFF is a standard file format for storing genomic features in a text file. 
GFF stands for Generic Feature Format. GFF files are plain text, 9 column, tab-delimited files. 
GFF databases also exist. They use a schema custom built to represent GFF data.
GFF is frequently used in GMOD for data exchange and representation of genomic data.
http://gmod.org/wiki/GFF3

Fields
Fields must be tab-separated. Also, all but the final field in each feature line must contain a value; "empty" columns should be denoted with a '.'

seqname - name of the chromosome or scaffold; chromosome names can be given with or without the 'chr' prefix. Important note: the seqname must be one used within Ensembl, i.e. a standard chromosome name or an Ensembl identifier such as a scaffold ID, without any additional content such as species or assembly. See the example GFF output below.
source - name of the program that generated this feature, or the data source (database or project name)
feature - feature type name, e.g. Gene, Variation, Similarity
start - Start position of the feature, with sequence numbering starting at 1.
end - End position of the feature, with sequence numbering starting at 1.
score - A floating point value.
strand - defined as + (forward) or - (reverse).
frame - One of '0', '1' or '2'. '0' indicates that the first base of the feature is the first base of a codon, '1' that the second base is the first base of a codon, and so on..
attribute - A semicolon-separated list of tag-value pairs, providing additional information about each feature.
"""





class gff_record(object):
	""" a parse for gff line """
	def __init__(self,line):
		self.line = line
		self._attrs = line.strip().split()
	def seqid(self):
		return self._attrs[0]
	def source(self):
		return self._attrs[1]
	def feature(self):
		return self._attrs[2]
	def start(self):
		return self._attrs[3]
	def end(self):
		return self._attrs[4]
	def score(self):
		return self._attrs[5]
	def strand(self):
		return self._attrs[6]
	def frame(self):
		return self._attrs[7]
	def attribute(self):
		return self._attrs[8]
	def region(self):
		return eval(self.end()) - eval(self.start())
	def out(self):
		return self.line





class GFF(object):
	""" Read GFF(general feature format) """
	def __init__(self,gffPath):
		self.gffPath = gffPath
		self.gff = self.all_gff()

		self._class = None
		self._mRNA = dict()
		self._CDS = dict()
		self.version3 = self.gff_version()
		self.geneid_pattern = re.compile(r'geneID=(?P<id>\S+);?')
		self.mRNA_pattern = re.compile(r'ID=(?P<id>\S+);')
		self.cds_pattern = re.compile(r'Parent=(?P<id>\S+);')


	def __iter__(self):
		""" Supports traversal with a for loop"""
		return iter(self._gff)
	def _handle(self):
		""" return a gff handle """
		if 'gz' in self.gffPath:
			fh= gzip.open(self.gffPath, 'rb')
		else:
			fh = open(self.gffPath, 'r')
		return fh

	def readGFF(self):
		""" Return gff file line by line through generator """
		for line in self._handle():
			yield line
	def all_gff(self):
		"""return a dict by rownum"""
		allgff=dict()
		for num,line in enumerate(self.readGFF()):
			if not line.startswith("#"):
				allgff[num]=gff_record(line)
		return allgff

	def gff_version(self):
		""" return gff version """
		#version = int(self._handle().readline().strip().split()[1])
		version = 3
		return True if version == 3 else False

	def annotation_chr_list(self):
		""" Return a chromosome list """
		chr_list = []
		for line in self.readGFF():
			if line.startswith("#"):
				continue
			else:
				chr_list.append(line.strip().split()[0])
		chromosomes = list(set(chr_list))
		chromosomes.sort(key=chr_list.index)
		return chromosomes

	def annotation_source(self):
		""" Return annotation source class list """
		source_list = []
		for line in self.readGFF():
			if line.startswith("#"):
				continue
			else:
				source_list.append(line.strip().split()[1])
		source = list(set(source_list))
		source.sort(key=source_list.index)
		return source

	def annotation_type(self):
		""" Return annotation type list """
		type_list = []
		for line in self.readGFF():
			if line.startswith("#"):
				continue
			else:
				type_list.append(line.strip().split()[2])
		types = list(set(type_list))
		types.sort(key=type_list.index)
		return types


	def gene_id_list(self):
		""" Return gene id list """
		geneid = []
		for line in self.readGFF():
			line = line.strip()
			match = self.geneid_pattern.search(line)
			if match:
				id = match.group("id")
				geneid.append(id)
		geneid_uniq = list(set(geneid))
		geneid_uniq_sort = sorted(geneid_uniq)
		return geneid_uniq_sort
	def geneCount(self):
		""" Return gene total number """
		geneid = self.gene_id_list()
		return len(geneid)

	def mrna_id_list(self):
		""" Return transcript id list """
		mrnaid = []
		for line in self.readGFF():
			line = line.strip()
			match = self.mRNA_pattern.search(line)
			if match:
				id = match.group("id")
				mrnaid.append(id)
		mrnaid_uniq = list(set(mrnaid))
		mrnaid_uniq_sort = sorted(mrnaid_uniq)
		return mrnaid_uniq_sort

	def mrnaCount(self):
		mrnaid = self.mrna_id_list()
		return len(mrnaid)

	def exonCount(self):
		exon = 0
		for line in self.readGFF():
			if line.startswith("#"):
				continue
			else:
				if line.strip().split()[2].lower() == "exon":
					exon += 1
		return exon


	def mRNA_fasta(self,genomefasta_path,mRNAfasta_path):
		genome = Fasta(genomefasta_path)
		genome.readFasta()
		genomeDict = genome._fasta
		forward = {}
		for record in self.all_gff().values():
			start,end = int(record.start()),int(record.end())
			if record.feature() == "mRNA":
				key = self.mRNA_pattern.search(record.attribute()).group("id")
				seq = genomeDict[record.seqid()].get_seq()[start-1:end]
				if record.strand() == "+":
					forward[key] = seq
				else:
					seq = seq.replace('A','{A}').replace('T','{T}').replace('C','{C}').replace('G','{G}')
					seq = seq.format(A='T', T='A', C='G', G='C')[::-1]
					forward[key] = seq

		mRNAfasta = open(mRNAfasta_path,'w')
		for key in forward:
			fa = fasta_record(key,forward[key])
			mRNAfasta.writelines(fa.fasta_parse())
		mRNAfasta.close()


	def cds_fasta(self,genomefasta_path,cds_outpath):
		genome = Fasta(genomefasta_path)
		genome.readFasta()
		genomeDict = genome._fasta
		forward = {}
		reverse = {}
		for record in self.all_gff().values():
			start,end = map(int,[record.start(),record.end()])
			if record.feature() == "mRNA":
				seq2 = ''
				seq3 = ''
			elif record.feature() == "CDS":
				key = self.cds_pattern.search(record.attribute()).group("id")
				out = genomeDict[record.seqid()].get_seq()[start-1:end]
				seq2 += out
				seq3 += out
				if record.strand() == "+":
					forward[key] = seq2
				else:
					reverse[key] = seq3
		for key in reverse.keys():
			seq = reverse[key]
			seq = seq.replace('A','{A}').replace('T','{T}').replace('C','{C}').replace('G','{G}')
			seq = seq.format(A='T', T='A', C='G', G='C')[::-1]
			forward[key] = seq
		CDS = open(cds_outpath,'w')
		for key in sorted(forward.keys()):
			fa = fasta_record(key,forward[key])
			CDS.writelines(fa.fasta_parse())
		CDS.close()

	def sortGFF(self):
		""" a """
		pass

	def gff2gtf(self,gtf_path):
		""" transform gff to gtf """
		GTF=open(gtf_path,'w')
		for line in self._handle():
			# skip comment lines that start with '#'
			if line[0] != "#":
				data = line.strip().split('\t') # split line into columns by tab
				ID = ''
				if data[2] == 'gene':
					ID = data[-1].split('ID=')[-1].split(';')[0]
				else:
					ID = data[-1].split('Parent=')[-1].split(';')[0]
				#modify last column
				data[-1] = 'gene_id "'+ID+'"; transcript_id "' + ID
				GTF.writelines('\t'.join(data))
		GTF.close()





import re,os
from random import sample
from itertools import groupby

"""
Author: Zhu Sitao
Date : 2018-3-21
Dest: a DNA fasta class; used in python 3.6
"""



BASES = ['T', 'C', 'A', 'G']
CODONS = [a + b + c for a in BASES for b in BASES for c in BASES]
AMINO_ACIDS = 'FFLLSSSSYY**CC*WLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG'
CODON_TABLE = dict(zip(CODONS, AMINO_ACIDS))


class fasta_record(object):
	""" a atom of fasta file """
	def __init__(self,id,seq,form="DNA"):
		self.id = id
		self.seq = seq
		self.form = form
	def get_id(self):
		return self.id
	def get_seq(self):
		return self.seq
	def reverse(self,seq):
		"""reverse fasta seq"""
		return seq[::-1]
	def complement(self,seq):
		"""reverse fasta seq"""
		COMPLEMENT_TRANS = str.maketrans('TAGCtagc', 'ATCGATCG')
		return seq.translate(COMPLEMENT_TRANS)
	def reverse_complement(self,seq):
		""" return reverse and complement seq"""
		return self.complement(self.reverse(seq))
	def translate(self,seq):
		seq = seq.lower().replace('\n', '').replace(' ', '')
		peptide = ''
		for i in range(0, len(seq), 3):
			codon = seq[i: i + 3]
			amino_acid = CODON_TABLE.get(codon, '!')
			if amino_acid != '!':  # end of seq
				peptide += amino_acid
		return peptide
	def fasta_parse(self,number=60):
		""" a parse for a fasta """
		seqOut = ''
		for i in range(1, len(self.seq) // number + 1):
			start = (i - 1) * number
			end = i * number
			seqOut += self.seq[start:end] + "\n"
		left = len(self.seq) % number
		remainder = self.seq[-left:] if left != 0 else ""
		totalOut = ">" + self.id + "\n" + seqOut + remainder + "\n"
		return totalOut





class Fasta(object):
	""" Fasta class """

	def __init__(self, filePath):
		self._fasta = dict()
		self.path = filePath
		#self.readFasta()

	def readFasta(self):
		""" Read Fasta file and load in a dict ,normal method
			使用groupby 将文本文件做成一个生成器，生成器没有把所有值存在内存中，而是在运行时生成的值，可以快速访问大文件。
			生成器你只能对其迭代一次。
		"""
		fh = open(self.path)
		faiter = (x[1] for x in groupby(fh, lambda line: line[0] == ">"))
		for header in faiter:
			header = header.__next__()[1:].strip()  # [1:] 为了去除 > 符号 header is a class 'itertools._grouper'
			header = header.split()[0]  # header.split()[0] 对名称进行简化，当前的做法是保存全部名称
			seq = "".join(s.strip() for s in faiter.__next__()) # faiter is a class 'generator'
			self._fasta[header] = fasta_record(header, seq)  # self._fasta[header] = seq.upper()
		return self._fasta


	def __iter__(self):
		""" Supports traversal with a for loop ,for ID loop """
		for record in self._fasta.values():
			yield record

	def __getitem__(self, index):
		""" Index one fasta ID """
		outPut = self._fasta[index]
		return outPut


	def __len__(self):
		""" Total sequence length of the fasta file """
		totalLength = 0
		for record in self._fasta.values():
			totalLength += len(record.seq)
		return totalLength

	def fasta_key(self):
		""" return a list of fasta name"""
		return self._fasta.keys()


	def fasta_sequence(self):
		""" return a list of sequence"""
		return self._fasta.values()


	def item_count(self):
		""" return numbers fasta names"""
		item_total_number = len(self.fasta_key())
		return item_total_number


	def std_out(self, number=60):
		""" printf sequence to stdout """
		for record in self.fasta_sequence():
			print(record.fasta_parse(number=number))


	def gc_rate(self, output_txt):
		""" Show each fasta GC and GC rate"""
		Stat = open(output_txt, 'w')
		Stat.writelines("ID\tGC\tGCrate(%)\tN\tNrate(%)\n")
		totalGC, totalN = 0, 0
		for record in self.fasta_sequence():
			seqIn = record.seq
			GC = seqIn.count("G") + seqIn.count("C")
			N = seqIn.count("N")
			totalGC += GC
			totalN += N
			GCrate = GC / len(seqIn)
			Nrate = N / len(seqIn)
			Stat.writelines("%s\t%d\t%.4f\t%d\t%.4f\n" % (ID, GC, GCrate, N, Nrate))
		totalGCrate = totalGC / len(self)
		totalNrate = totalN / len(self)
		Stat.writelines("total\t%d\t%.4f\t%d\t%.4f" % (totalGC, totalGCrate, totalN, totalNrate))
		Stat.close()


	def stat_length(self):
		""" Statistic of each id length """
		# LenDict = defaultdict(int)
		LenDict = dict()
		for ID,record in self._fasta.items():
			LenDict[ID] = len(record.seq)
		return LenDict


	@classmethod
	def basename(cls):
		""" class method """
		return os.path.basename(cls)


	@staticmethod
	def print_working_directory():
		""" static method ,no default parameter """
		return os.getcwd()


	def _max_min(self, type):
		sort_dict = sorted(self.stat_length().items(), key=lambda d: d[1], reverse=True)
		if type == 'max':
			max_item = sort_dict[0]
			return max_item
		elif type == 'min':
			min_item = sort_dict[self.item_count() - 1]
			return min_item
		else:
			return "%s not max or min" % type


	def extract_item(self, type, outfile):
		""" extract max or min length fasta item"""
		if type.lower() not in ['max', 'min']:
			raise KeyError(str(type) + " not contain,should be min or max")
		with open(outfile, 'w') as OUT:
			key, value = self._max_min(type)
			record = self._fasta[key]
			if type == 'max':
				OUT.writelines(record.fasta_parse())
			elif type == 'min':
				OUT.writelines(record.fasta_parse())
			else:
				print("%s not correct,should be max or min" % type)


	def random_sample(self, number, outpath):
		""" ramdom sample from the all fasta file"""
		OUT = open(outpath, 'w')
		key_list = self.fasta_key()
		sample_list = sample(key_list, number)
		for key in sample_list:
			record = self._fasta[key]
			OUT.writelines(record.fasta_parse())
		OUT.close()







#coding:utf-8

"""
Author : Sitao Zhu
Date: 2019-4-18
Desc:
# genome file is the location of genome file;
# sample list file is like this:
# BEGIN
# sample1    population1
# sample2    population1
# sample3    population2
# END
# the sample name should be same as which in vcf file.
# Usage: python vcf2dadi.py -r <genome file> -f <vcf file> -l <sample list file>

"""


import os
import re
import getopt
from format.fasta import Fasta



def parse_sample(sample_list):
    """ put sample list file into a dict """
    samples={}
    with open(sample_list,'r') as F:
        for line in F.readline():
            if line.startswith('#'):
                continue
            else:
                fields = line.strip().split()
                samples[fields[0]] = fields[1]
    return samples

def parse_vcf(vcf,samples):
    """ put vcf file into a dict """
    pattern = re.compile(r'/^(.)\/(.):/')
    record = {}
    vcf = {}
    pop = {}
    with open(vcf,'r') as F:
        for line in F.readline():
            line = line.strip()
            if line.startswith('##'):continue
            if line.startswith('#CHROM'):
                fields = line.split()
                for i in range(9,len(fields)):
                    if fields[i] not in sample.keys():
                        continue
                    else:
                        record[i] = samples[fields[i]]
            a = line.split()
            chrom,pos,ref,alt = a[0],a[1],a[3],a[4]
            if ',' in alt:continue
            vcf[chrom] = {pos: {'ref': ref}}
            vcf[chrom] = {pos: {'alt': alt}}


            for key in record.keys():
                indv = record[key]
                # vcf[chrom][pos] = {indv: {'a1', 0}}
                # vcf[chrom][pos] = {indv: {'a2', 0}}
                pop[indv] = 1
                geno = a[key]
                match = pattern.match(geno)
                if match:
                    tempa = match.group(1)
                    tempb = match.group(2)
                    a1,a2 = 0,0
                    if tempa == '.' or tempb == '.':
                        a1 = 0
                        a2 = 0
                    elif int(tempa) + int(tempb) == 1:
                        a1 = 1
                        a2 = 1
                    elif int(tempa) + int(tempb) == 2:
                        a1 = 0
                        a2 = 2
                    elif int(tempa) + int(tempb) == 0 :
                        a1 = 2
                        a2 = 0
                    if not vcf[chrom][pos][indv]:
                        vcf[chrom][pos] = {indv:{'a1':a1}}
                        vcf[chrom][pos] = {indv:{'a2':a2}}
                    else:
                        vcf[chrom][pos][indv]['a1'] += a1
                        vcf[chrom][pos][indv]['a2'] += a2

    with open("list.data",'w') as F:
        title = "NAME\tOUT\tAllele1"
        for key in sorted(pop.keys()):
            title += "\t%s"%(key)
        title += "\tAllele2"
        for key in sorted(pop.keys()):
            title += "\t%s"%(key)
        title += "\tGene\tPostion\n"
        F.writelines(title+"\n")
        fasta = Fasta(fasta_path)



# -*- coding: UTF-8 -*-
import os

def readme():
    with open("README.rst") as f:
        return f.read()

from setuptools import setup 
setup(
    name='promoterExtract',
    version='0.9.5.7',
    keywords='promoter',
    description='Extract promoter sequence for biologists',
    long_description_content_type='text/markdown',
    long_description=readme(),
    entry_points = {'console_scripts': ['get_promoter=promoterExtract.command_line:main']},
    author='zhusitao',
    author_email='zhusitao1990@163.com',
    url='https://github.com/SitaoZ/promoterExtract.git',
    include_package_data=True,
    packages=['promoterExtract'],
    license='MIT',
    install_requires = ['argparse>=1.1', 
                        'pandas>=1.2.4', 
                        'gffutils>=0.10.1',
                        'setuptools>=49.2.0',
                        'biopython>=1.78'])



import argparse

class Parameter():
    def __init__(self):
        self._parser = argparse.ArgumentParser()
    
    def parse(self):
        subparsers = self._parser.add_subparsers(help='sub-command help')
        # create subcommand 
        parser_create = subparsers.add_parser('create', help='create help')
        parser_create.add_argument('-g', '--gff', type=str, help='genome annotation file')
        parser_create.set_defaults(func='create')
        # extract subcommand
        parser_extract = subparsers.add_parser('extract', help='extract help')
        parser_extract.add_argument('-l', '--length', type=int, help='promoter length before TSS')
        parser_extract.add_argument('-u', '--utr_head', type=int, help='length after TSS')
        parser_extract.add_argument('-f', '--genome', type=str, help='genome fasta')
        parser_extract.add_argument('-g', '--gff', type=str, help='genome annotation file')
        parser_extract.add_argument('-o', '--output', type=str, help = 'output csv file path')
        parser_extract.add_argument('-v', '--version', help = 'promoterExtract version', action = "store_true")
        parser_extract.set_defaults(func='extract')
        #
        args = self._parser.parse_args()
        #args.func(args)

        #if args.version:
        #    print("promorerExtract version 0.9.3")
        #    exit(1)

        #self.length = args.length
        #self.utr_head = args.utr_head
        #self.genome = args.genome
        self.gff = args.gff
        #self.output = args.output
        
        #if self.length is None:
        #    raise  Exception("Error: promoter length is required")
        #elif self.utr_head is None:
        #    raise Exception("Error: utr length is required")
        #elif self.genome is None:
        #    raise Exception("Error: genome fasta is required")
        #elif self.gff is None:
        #    raise Exception("Error: gff is required")
        #elif self.output is None:
        #    raise Exception("Error: output is required")

        return self
